**********************************************************
* This script builds the file used in the event studies
* Later routines refer to a Stata port of the output file 
* titled eventspells.dta
**********************************************************

*eventstudy_x10.sas ;
*for_event_study_males and for_event_study_females previously created (and in sas7bdat format);
*vars:  id firmid year y age educ female ;

options ls=120 nocenter nofmterr;
libname here '.';
libname store '/var/tmp';

data work.zero;
set store.for_event_study_males store.for_event_study_females;
run;

proc means;
class female;
title1 'pooled m+f connected sets';
run;


proc sql;
create table work.fy as
select firmid, year,
mean(y) as meanwage,
mean(educ) as meaneduc,
count(y) as emp,
sum(female) as empf,
sum((1-female)) as empm,
divide ( sum(y*(1-female)) , calculated empm ) as meanwagem,
divide ( sum(y*female), calculated empf  ) as meanwagef
from work.zero
group by firmid, year;
quit;

proc means data=fy;
title1 'fy data set - by year';
class year;

proc sql;
create table work.fy2 as
select *,
mean(empm) as meanempm,
mean(empf) as meanempf,
mean(meaneduc) as ameaneduc,
calculated meanempf=0 as nofemales,
calculated meanempm=0 as nomales,
mean(emp) as meanemp
from work.fy
group by firmid
order by firmid, year;
quit;

proc datasets lib=work;
delete fy;
quit;


proc means data=work.fy2;
title1 'fy data set - by year';
class year;

proc means data=work.fy2;
title1 'fy data set - by year all male firms';
where (nofemales=1);
class year;

proc means data=work.fy2;
title1 'fy data set - by year all female firms';
where (nomales=1);
class year;

proc means data=work.fy2;
title1 'fy data set - by year EVER mixed firms';
where (nomales=0 and nofemales=0);
class year;



proc sort data=work.zero;
by firmid year;
run;


data work.one;
merge work.zero (in=in1) work.fy2;
by firmid year;

*create 3 leave out means;

if emp>1 then lom=(meanwage-y/emp)*emp/(emp-1);
else lom=.;

if emp>1 then lomed=(meaneduc-educ/emp)*emp/(emp-1);
else lomed=.;


if female=0 then do;
 if empm>1 then lomm=(meanwagem-y/empm)*empm/(empm-1);
  else lomm=.;
 if empf>0 then lomf=meanwagef; 
  else lomf=.;
end; 

if female=1 then do;
 if empm>0 then lomm=meanwagem; 
  else lomm=.;
 if empf>1 then lomf=(meanwagef-y/empf)*empf/(empf-1);
  else lomf=.;
end; 

run;


proc corr data=work.one;
title1 'co-worker wages';
var y meanwage lom lomm lomf educ meaneduc lomed ameaneduc;
run;

proc corr data=work.one;
title1 'co-worker wages by gender';
where ( female=0);
var y meanwage lom lomm lomf educ meaneduc lomed;
run;

proc corr data=work.one;
title1 'co-worker wages by gender';
where ( female=1);
var y meanwage lom lomm lomf educ meaneduc lomed;
run;

proc corr data=work.one;
title1 'co-worker wages - mixed firms';
where (nomales=0 and nofemales=0);
var y meanwage lom lomm lomf educ meaneduc lomed;
run;

proc corr data=work.one;
title1 'co-worker wages - mixed firms - men';
where (female=0 and nomales=0 and nofemales=0);
var y meanwage lom lomm lomf educ meaneduc lomed;
run;

proc corr data=work.one;
title1 'co-worker wages - mixed firms - women';
where (female=1 and nomales=0 and nofemales=0);
var y meanwage lom lomm lomf educ meaneduc lomed;
run;


***from now on, mixed firms;

data work.one;
set work.one;
if nomales=0 and nofemales=0;


data work.r1;
set work.one (keep=lom where=(lom ne .) );
run;


proc rank data=work.r1 out=work.r2 groups=4;
var lom;
ranks qlom;
run;

proc datasets lib=work;
delete r1;
run;

proc freq data=work.r2;
title2 'ranking procedure';
tables qlom / missing;
run;


proc means data=work.r2;
where (qlom=0);
output out=work.p1
max(lom)=cutpoint1;
run;

proc means data=work.r2;
where (qlom=1);
output out=work.p2
max(lom)=cutpoint2;
run;

proc means data=work.r2;
where (qlom=2);
output out=work.p3
max(lom)=cutpoint3;
run;

proc datasets lib=work;
delete r2;
run;


data _NULL_;
set work.p1;
call symput('cut1', cutpoint1);
run;

data _NULL_;
set work.p2;
call symput('cut2', cutpoint2);
run;

data _NULL_;
set work.p3;
call symput('cut3', cutpoint3);
run;




data work.one;
set work.one;

singleton=(lom = .);

if singleton=0 then do;
 if lom<=&cut1. then qlom=1;
 else if lom<=&cut2. then qlom=2;
 else if lom<=&cut3. then qlom=3;
 else qlom=4;
end;

else do;
 qlom=0;
 end;
run;

proc freq;
tables qlom*singleton / missing;
run;
   
proc sort data=work.one;
by id year;
run;


data work.two;
set work.one;
by id year;

retain tenure startyr jobnum;

jobchange=0;
lagfirmid=lag(firmid);
lag2firmid=lag2(firmid);
lag1y=lag(y);
lag2y=lag2(y);
lag3y=lag3(y);
lag4y=lag4(y);

lagqlom=lag(qlom);

if first.id then do;
 lagfirmid=.;
 tenure=1;
 startyr=year;
 jobnum=1;
 end;

else do;
 if firmid=lagfirmid then tenure=tenure+1;
 else do;
    jobchange=1;
    tenure=1;
    jobnum=jobnum+1;
    startyr=year;
    end;
 end;
run;

proc datasets lib=work;
delete one;
quit;


proc print data=work.two (obs=20);
title2 'test of assigning lagged wages and job change indicator';
var id year firmid lagfirmid jobnum startyr tenure jobchange y lag1y;
run;


proc sort data=work.two;
by id descending year;
run;

data work.one;
set work.two;
by id descending year;
retain endyr yearstogo;
nextfirmid=lag(firmid);
next2firmid=lag2(firmid);

lead1y=lag(y);
lead2y=lag2(y);
lead3y=lag3(y);
lead4y=lag4(y);

if first.id then do;
  nextfirmid=.;
  endyr=year;
  yearstogo=1;
  end;
 
else do;
 if firmid=nextfirmid then yearstogo=yearstogo+1;
 else do; 
   endyr=year;
   yearstogo=1;
   end;
 end;

run;



proc print data=work.one (obs=20);
title2 'test of assigning lead wages and nextfirmid';
var id year firmid nextfirmid endyr yearstogo;
run;



proc sql;
create table work.two as
  select *,
  min(year) as firstyr,
  max(year) as lastyr,
  max(jobnum) as numjobs
  from work.one
  group by id
  order by id, year;
quit;

proc datasets lib=work;
delete one;
quit;


data work.one;
set work.two;
by id year;

lagtenure=lag(tenure);
lagemp=lag(emp);
lagmeanemp=lag(meanemp);
lagmeaneduc=lag(meaneduc);
lagameaneduc=lag(ameaneduc);
laglom=lag(lom);
laglomm=lag(lomm);
laglomf=lag(lomf);
laglomed=lag(lomed);


if first.id=1 then do;
 lagtenure=.;
 lagemp=.;
 lagmeanemp=.;
 lagmeaneduc=.;
 lagameaneduc=.;
 laglom=.;
 laglomed=.;
 end;



event=0;
stayspell=0;


if year=startyr and jobnum>=2 then do;
 event=1;
 numpre=lagtenure;
 numpost=yearstogo;
 end;

if lagfirmid=firmid and lag2firmid=firmid and nextfirmid=firmid and next2firmid=firmid then 
 stayspell=1;

run;

proc datasets lib=work;
delete two;
quit;


proc means data=work.one;
title2 'means of data with event variable and leads/lags assigned';
run;




data events;
set work.one;
if event=1 and numpre>=2 and numpost>=2;
if lagqlom ne 0 and qlom ne 0;

dw=lead1y-lag2y;
yr=year-2;
agesq=age*age/100;
event=1;

keep id year yr female educ age agesq lagqlom qlom lom laglom    	
     lag2y lag1y y lead1y dw laglomed lomed event lagfirmid firmid;

run;


proc means;
title2 'events with 2+ years pre and 2+ years post - exclude singleton firms';
run;

proc means;
class female lagqlom;
run;


data stayspells;
set work.one;
if stayspell=1 and lagqlom ne 0;

dw=lead1y-lag2y;
yr=year-2;
agesq=age*age/100;
event=0;

keep id year yr female educ age agesq lagqlom qlom lom laglom    	
     lag2y lag1y y lead1y dw laglomed lomed event lagfirmid firmid;
run;

proc means;
title2 'stayspells with 2+ years pre and 2+ years post - exclude singleton firms';
run;

proc means;
class female lagqlom;
run;





data one;
set events stayspells;

if event=1 then dws=.;
else dws=dw;


proc freq;
title2 'merged events and stayspells';
tables female*event female*yr*event female*lagqlom*event / missing;
run;

proc means;
class event;
var dw dws female educ;



proc glm;
class yr educ female;
model dws=yr*female educ*female educ*female*age educ*female*agesq / solution;
output out=work.two
p=dwpred ;
run;

proc means;
class female event;

proc means;
class female event lagqlom;

data work.two;
set work.two;
dwr=dw-dwpred;

if firmid=lagfirmid then samefirm=1;
else samefirm=0;

proc freq;
tables event*samefirm / missing;

proc means;
class event;
var dw dws dwpred dwr female educ;



proc summary data=work.two nway;
class female lagqlom qlom;
where (event=1);
var lag2y lag1y y lead1y dw dwr laglom lom laglomed lomed;
output out=here.events4r
mean=
n(y)=nevents;
run;


proc print data=here.events4r;
var female lagqlom qlom nevents lag2y lag1y y lead1y dw dwr laglom lom laglomed lomed; 
run;



data store.eventspells;
set work.two;


proc means;
class female event;


